from config import settings
from utils import wallpaper
from utils.BGWatcher import BGWatcher

try:
    from utils import _ewmh as ewmh
except ImportError:
    print "Couldn't load EWMH support. Some window flags will not work."

import gtk
import gobject


_BGWATCHER = BGWatcher()

_MOOSE_DELAY = 100


# this gives us support for real transparency on the freedesktop.org Xserver;
# it's a bit messed up, though, because GTK isn't prepared for it... :(
if (settings.translucent):
    try:
        # get the ARGB visual for an alpha channel
        visual = gtk.gdk.visual_get_best_with_depth(32)
        cmap = gtk.gdk.Colormap(visual, gtk.TRUE)
        gtk.widget_push_colormap(cmap)

    except:
        # nope, you don't have that l33t Xserver, sorry! :/
        print "Translucency is not available on this system."
        # I don't switch back to pseudo-transparency here because then users
        # might think that --translucent worked where it didn't.


#
# Class for transparent windows.
#
class GlassWindow(gtk.Window):

    #
    # Constructor.
    #
    def __init__(self, wintype = gtk.WINDOW_TOPLEVEL):

        # handler for transparency updates
        self.__update_handler = None

        # the desktop background
        self.__bg_root = None

        # remember the current window position to detect movements
        self.__position = (0, 0)
        self.__size = (0, 0)

        gtk.Window.__init__(self, wintype)

        # welcome to the future!
        if (not settings.translucent):
            # mommy, my xserver doesn't have translucency... :(
            self.connect("configure-event", self.__on_configure)
            _BGWATCHER.add_observer(self.__bg_observer)

            self.__bg_image = gtk.gdk.Pixbuf(0, 1, 8, 10, 10)
            self.__bg_image.fill(0xffffff)

        else:
            # real men don't need wimpy pseudo-transparency... 8)
            self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))



    #
    # Observer method for the background.
    # Connect this method to the BG watcher.
    #
    def __bg_observer(self, src, cmd):

        self.__update_bg()


    #
    # Updates the background for transparency.
    #
    def __update_bg(self):

        if (self.__update_handler):
            gobject.source_remove(self.__update_handler)
        if (_MOOSE_DELAY):
            self.__update_handler = gtk.timeout_add(_MOOSE_DELAY,
                                                    self.__updater)
        else:
            self.__updater()



    def __updater(self):

        if (not self.window): return

        x, y = self.window.get_origin()
        width, height = self.window.get_size()
        if (width > 1 and height > 1):
            self.__capture_bg(x, y, width, height)


    #
    # Captures the background to create transparency.
    #
    def __capture_bg(self, x, y, width, height):

        wallpaper.get_wallpaper(self, x, y, width, height)
        self.queue_draw()


    #
    # Reacts on moving the window.
    #
    def __on_configure(self, src, event):

        pos = self.window.get_origin()
        size = self.window.get_size()
        if (pos != self.__position or size != self.__size):
            self.__position = pos
            self.__size = size
            self.__update_bg()



    #
    # Sets the BELOW window flag.
    #
    def _set_flag_below(self, value):

        if (not self.__get_window_manager() in ["Enlightenment"] and
              not self.get_property("visible")):
            gtk.timeout_add(500, self._set_flag_below, value)
            
        ewmh.set_below(self.window, value)
        #return gtk.TRUE

    #
    # Sets the ABOVE window flag.
    #
    def _set_flag_above(self, value):

        if (not self.__get_window_manager() in ["Enlightenment"] and
            not self.__get_window_manager().startswith("Xfwm4") and
              not self.get_property("visible")):
            gtk.timeout_add(500, self._set_flag_above, value)

        ewmh.set_above(self.window, value)
        

    #
    # Sets the STICKY window flag.
    #
    def _set_flag_sticky(self, value):

        if (value): self.stick()
        else: self.unstick()



    #
    # Sets the STICKY window flag.
    #
    def _set_flag_sticky(self, value):

        if (value): self.stick()
        else: self.unstick()


    #
    # Sets the MANAGED window flag.
    #
    def _set_flag_managed(self, value):

        wm = self.__get_window_manager()
        if (value):
            self.set_property("skip-taskbar-hint", 0)
            self.set_property("skip-pager-hint", 0)
            ewmh.set_type_dock(self.window, False)
        else:
            self.set_property("skip-taskbar-hint", 1)
            self.set_property("skip-pager-hint", 1)
            if (wm == "Metacity"):
                ewmh.set_type_dock(self.window, True)


    #
    # Sets the DECORATED window flag.
    #
    def _set_flag_decorated(self, value):

        if (value): self.set_decorated(gtk.TRUE)
        else: self.set_decorated(gtk.FALSE)



    #
    # Returns the name of the running EWMH compliant window manager or "".
    #
    def __get_window_manager(self):

        try:
            root = gtk.gdk.get_default_root_window()
            ident = root.property_get("_NET_SUPPORTING_WM_CHECK",
                                      "WINDOW")[2][0]
            win = gtk.gdk.window_foreign_new(long(ident))
            name = win.property_get("_NET_WM_NAME",
                                      "UTF8_STRING")[2]

        except:
            name = ""

        #print "Window Manager:", name
        return name
