from main import VERSION

import gtk
import os


_CREDITS = [("Version " + VERSION,
             [u"Copyright \xa9 2003 - 2004, The gDesklets Team"]),

            ("Core Programming:",
             ["Martin Grimme", "Christian Meyer", "Jesse Andrews",
              u"Beno\xeet Dejean"]),

            ("Architecture Design:",
             ["Martin Grimme", "Christian Meyer"]),

            ("GNOME Integration:",
             ["Christian Neumair", "Martin Grimme", "Sebastien Bacher",
              "Christian Kellner"]),

            ("Artwork:",
             ["Johannes Rebhan"]),

            ("Website:",
             ["Luke Stroven"])
            ]


#
# Class for a fancy "About" dialog.
#
class AboutDialog(gtk.Window):

    def __init__(self, path):

        self.__is_stopped = True
        self.__scroller_values = ()
        

        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        self.set_position(gtk.WIN_POS_CENTER)
        self.set_resizable(False)
        self.set_title("About gDesklets")
        self.connect("button-press-event", self.__on_close)
        self.connect("key-press-event", self.__on_close)
        self.add_events(gtk.gdk.BUTTON_PRESS_MASK)

        fixed = gtk.Fixed()
        self.add(fixed)

        img = gtk.Image()
        img.set_from_file(os.path.join(path, "about.png"))
        fixed.put(img, 0, 0)

        w, h = img.size_request()

        scroller = gtk.Fixed()
        scroller.set_size_request(w, h - 32)
        fixed.put(scroller, 0, 32)

        text = ""
        for header, body in _CREDITS:
            text += "<b>" + header + "</b>\n\n"
            text += "\n".join(body)
            text += "\n\n\n\n"
        text = "<big>" + text.strip() + "</big>"
        credits = gtk.Label(text)
        credits.set_use_markup(True)
        credits.set_justify(gtk.JUSTIFY_CENTER)

        lbl_width, lbl_height = credits.size_request()
        scroller.put(credits, (w - lbl_width) / 2, h)

        self.__scroller = scroller
        self.__credits = credits

        self.__scroller_values = (h - 32, -lbl_height, (w - lbl_width) / 2)


    def __scroll(self, y):

        begin, end, x = self.__scroller_values
        self.__scroller.move(self.__credits, x, y)
        y -= 1

        if (y < end): y = begin

        if (not self.__is_stopped):
            gtk.timeout_add(40, self.__scroll, y)

        return False


    def __on_close(self, *args):

        self.__is_stopped = True
        self.hide()


    def show(self):

        if (not self.__is_stopped): return
        
        self.show_all()

        self.__is_stopped = False
        begin, end, x = self.__scroller_values
        gtk.timeout_add(0, self.__scroll, begin)
