#
# Conversion functions between units.
#

from datatypes import *

import gtk


UNIT_CM = "cm"
UNIT_IN = "in"
UNIT_PERCENT = "%"
UNIT_PT = "pt"
UNIT_PX = "px"


#
# Returns the screen resolution in DPI.
#
def _guess_dpi():

    height = gtk.gdk.screen_height()
    height_inch = gtk.gdk.screen_height_mm() / 25.4  # 1 inch is 25.4 mm
    dpi = height / height_inch

    return dpi


def cm_to_pixels(value): return int(value * _cm2in_scale * _in2px_scale)
def inches_to_pixels(value): return int(value * _in2px_scale)
def points_to_pixels(value): return int(value * _pt2in_scale * _in2px_scale)

def cm_to_points(value): return int(value * _cm2in_scale * _in2pt_scale)
def inches_to_points(value): return int(value * _in2pt_scale)
def pixels_to_points(value): return int(value * _px2in_scale * _in2pt_scale)

def pixels_to_all(value):

    inches = value * _px2in_scale
    cms = inches * _in2cm_scale
    pts = inches * _in2pt_scale
    return (cms, inches, pts)


def units_to_pixels(value, unit):

    if (unit == UNIT_PX): return int(value)
    elif (unit == UNIT_CM): return cm_to_pixels(value)
    elif (unit == UNIT_IN): return inches_to_pixels(value)
    elif (unit == UNIT_PT): return points_to_pixels(value)
    else: raise ValueError("Invalid unit: %d" % unit)


def units_to_points(value, unit):

    if (unit == UNIT_PX): return pixels_to_points(value)
    elif (unit == UNIT_CM): return cm_to_points(value)
    elif (unit == UNIT_IN): return inches_to_points(value)
    elif (unit == UNIT_PT): return value
    else: raise ValueError("Invalid unit: %d" % unit)


# resolution of the screen
_dpi = _guess_dpi()

# scaling factors
_in2pt_scale = 72.0
_pt2in_scale = 1 / _in2pt_scale
_in2px_scale = float(_dpi)
_px2in_scale = 1 / _in2px_scale
_in2cm_scale = 2.54
_cm2in_scale = 1 / _in2cm_scale
